' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_LargeFont_001.spin                                                                       *
' *                                                                                                     *
' *******************************************************************************************************

OBJ

  tv_LargeFont_Driver           : "AiChip_LargeFont_Driver_001"

CON

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Driver Attributes                                                                               |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  COLUMNS                       = tv_LargeFont_Driver#COLUMNS
  ROWS_PAL                      = tv_LargeFont_Driver#ROWS_PAL
  ROWS_NTSC                     = tv_LargeFont_Driver#ROWS_NTSC
  COLOURS                       = 16
  
  FONT_CHARS                    = 256

  ROWS_MAX                      = ROWS_NTSC #> ROWS_PAL
  
  SCREEN_SIZE_CHARS             = COLUMNS * ROWS_MAX

  TV_PARAMS_SIZE                = 14

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Characters for drawing boxes                                                                    |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  CHR_TOP_LEFT                  = 159
  CHR_TOP_MID                   = 144
  CHR_TOP_RIGHT                 = 158
  CHR_LFT_BAR                   = 145
  CHR_BOT_LEFT                  = 157
  CHR_BOT_RIGHT                 = 156

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Characters for drawing logic analyser traces                                                    |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  CHR_BIT_0_TO_0                = 129
  CHR_BIT_0_TO_1                = 254
  CHR_BIT_1_TO_0                = 255
  CHR_BIT_1_TO_1                = 134

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Mode bit settings - Must be the same as AiChip_TvText_001.spin                                  |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  TV_MODE_NTSC                  = %0000_0000
  TV_MODE_PAL                   = %0000_0001

  TV_MODE_LARGE_FONT            = %0000_0000
  TV_MODE_SMALL_FONT            = %0000_0010

  TV_MODE_FAST_UPDATE           = %0000_0000
  TV_MODE_FLICKER_FREE          = %0000_0100

  TV_MODE_COLOR                 = %0000_0000
  TV_MODE_COLOUR                = %0000_0000
  TV_MODE_MONOCHROME            = %0000_1000
  
  TV_MODE_INTERLACED            = %0000_0000
  TV_MODE_NON_INTERLACED        = %0001_0000

  TV_MODE_COMPOSITE             = %0000_0000
  TV_MODE_BASEBAND              = %0000_0000
  TV_MODE_BROADCAST             = %0010_0000

  TV_MODE_FONT_0                = %0000_0000
  TV_MODE_FONT_1                = %0100_0000

  TV_MODE_RUNNING               = %1000_0000
  
VAR

  word screenBuffer[ SCREEN_SIZE_CHARS ]
  long colorPalette[ COLOURS * 2 ]
  byte rows
  word lastRow

  byte mode
  byte ucharHasBeenAligned

  long tv_status     '0/1/2 = off/invisible/visible              read-only   (14 longs)
  long tv_enable     '0/non-0 = off/on                           write-only
  long tv_pins       '%pppmmmm = pin group, pin group mode       write-only
  long tv_mode       '%tccip = tile,chroma,interlace,ntsc/pal    write-only
  long tv_screen     'pointer to screen (words)                  write-only
  long tv_colors     'pointer to colors (longs)                  write-only
  long tv_cols       'horizontal tiles                           write-only
  long tv_rows       'vertical tiles                             write-only
  long tv_hx         'horizontal tile expansion                  write-only
  long tv_vx         'vertical tile expansion                    write-only
  long tv_ho         'horizontal offset                          write-only
  long tv_vo         'vertical offset                            write-only
  long tv_broadcast  'broadcast frequency (Hz)                   write-only
  long tv_auralcog   'aural fm cog                               write-only

' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setBasePin, setMode, ptrToPalette ) | ptrToParams

  if mode <> setMode

    Close
    
    mode := setMode

    SetPalette(ptrToPalette)
    if not ucharHasBeenAligned
      longmove(@UCHAR & $FF80,@UCHAR,32*2)
      ucharHasBeenAligned := true
    if mode & TV_MODE_PAL
      ptrToParams := @PAL_PARAMS
    else
      ptrToParams := @NTSC_PARAMS
    longmove(@tv_status, ptrToParams, TV_PARAMS_SIZE)
    tv_pins := (setBasePin & $38) << 1 | (setBasePin & 4 == 4) & %0101
    tv_screen := @screenBuffer
    tv_colors := @colorPalette
    rows := tv_rows
    lastRow := (rows-1) * COLUMNS
    if mode & TV_MODE_MONOCHROME
      tv_mode ^= %1100
    if mode & TV_MODE_NON_INTERLACED
      tv_mode &= ! %0010
    if mode & TV_MODE_BROADCAST
      tv_pins |= %0001
    Cls
    tv_LargeFont_Driver.Open(@tv_status)

  return mode
  
PUB Close

  if mode
    tv_LargeFont_Driver.Close
    mode := 0

' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB Cls

  wordfill(@screenBuffer, $220, ROWS_MAX * COLUMNS)

PUB Scroll

  if mode & TV_MODE_FLICKER_FREE
    WaitForSynch
  wordmove(@screenBuffer, @screenBuffer[COLUMNS], lastRow)  ' Scroll lines
  wordfill(@screenBuffer[lastRow], $220, COLUMNS)           ' Clear new line

PUB PokeChar( row, col, colour, c )

  colour //= COLOURS
  c &= $FF
  
  if mode & TV_MODE_FLICKER_FREE
    WaitForSynch

  if c => 254
    c := (colour << 1 ) << 10 + ( ( @uchar & $FF80 ) >> 6 ) + ( ( c - 254 ) << 1 )
  else
    c := (colour << 1 + c & 1) << 10 + $200 + c & $FE

  screenBuffer[row * COLUMNS + col] := c

PRI WaitForSynch

  repeat while tv_status == 1
  repeat while tv_status == 2

' *******************************************************************************************************
' *                                                                                                     *
' *     Logic Analyser Trace Handling Routines                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetLogicAnalyserBit(thisBit,nextBit)

  result := LookUpZ ( ( ( thisBit >> 30 ) & 2 ) | ( nextBit >> 31 ) : CHR_BIT_0_TO_0, CHR_BIT_0_TO_1, CHR_BIT_1_TO_0, CHR_BIT_1_TO_1 )

' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColourCount

  result := COLOURS

PUB GetColumnCount

  result := COLUMNS
    
PUB GetRowCount

  result := rows

' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette Handling for Video Drivers                                                       *
' *                                                                                                     *
' *******************************************************************************************************

PUB SetPalette( ptrToPalette ) | i, back

  back := byte[ptrToPalette]
  repeat i from 0 to COLOURS-2
    SetPaletteColour(i,byte[ptrToPalette+i+1],back)
  SetPaletteColour(COLOURS-1,byte[ptrToPalette+1],byte[ptrToPalette+COLOURS])

PUB SetPaletteColour( colorIdx, fore, back )

   colorPalette[colorIdx << 1]     := fore << 24 + back << 16 + fore << 8 + back
   colorPalette[colorIdx << 1 + 1] := fore << 24 + fore << 16 + back << 8 + back

DAT

' *******************************************************************************************************
' *                                                                                                     *
' *     Low-Level driver Parameters                                                                     *
' *                                                                                                     *
' *******************************************************************************************************

NTSC_PARAMS   long    0               'status
              long    1               'enable
              long    0               'pins
              long    %10010          'mode
              long    0               'screen
              long    0               'colors
              long    COLUMNS         'ht
              long    ROWS_NTSC       'vt
              long    4               'hx
              long    1               'vx
              long    0               'ho
              long    0               'vo
              long    0               'broadcast
              long    0               'auralcog

PAL_PARAMS    long    0               'status
              long    1               'enable
              long    0               'pins
              long    %10011          'mode
              long    0               'screen
              long    0               'colors
              long    COLUMNS         'ht
              long    ROWS_PAL        'vt
              long    5               'hx
              long    1               'vx
              long    0               'ho
              long    0               'vo
              long    66_750_00       'broadcast
              long    0               'auralcog

' *******************************************************************************************************
' *                                                                                                     *
' *     User-Defined Characters                                                                         *
' *                                                                                                     *
' *******************************************************************************************************

              LONG 0[31] 'alignment padding

UCHAR         LONG %%0000_0000_0000_0000[5]   '             *
              LONG %%1100_0000_0000_0000      '            *
              LONG %%1110_0000_0000_0000      '            *
              LONG %%1110_0000_0000_0000      '            *
              LONG %%0110_0000_0000_0000[16]  '            *
              LONG %%0111_0000_0000_0000      '            *
              LONG %%0111_1111_1111_1111      '            *
              LONG %%0011_1111_1111_1111      '            *
              LONG %%0000_0000_0000_0000[5]   ' ***********

              LONG %%0000_0000_0000_0000[5]   ' ***********
              LONG %%0011_1111_1111_1111      '            *
              LONG %%0111_1111_1111_1111      '            *
              LONG %%0111_0000_0000_0000      '            *
              LONG %%0110_0000_0000_0000[16]  '            *
              LONG %%1110_0000_0000_0000      '            *
              LONG %%1110_0000_0000_0000      '            *
              LONG %%1100_0000_0000_0000      '            *
              LONG %%0000_0000_0000_0000[5]   '             *

